header "chipset" /{

#include <linux/ctype.h>

#define CHIPSET_none      0
#define CHIPSET_82c200    1
#define CHIPSET_i82527    2
#define CHIPSET_emu       3
#define CHIPSET_emscpc    4
#define CHIPSET_pccan     5



typedef struct {
  int (* ChipReset)  (int minor);
  int (* SetTiming)  (int board,int baud);
  int (* StartChip)  (int board);
  int (* StopChip)   (int board);
  int (* SetMask)    (int board, int arg);
  int (* SetOMode)   (int board, int arg);
  int (* SendMessage)(int board, canmsg_t *tx, int wait);
  int (* GetMessage) (int board, canmsg_t *rx );
  

  void (* Interrupt) ( int irq, void *unused, struct pt_regs *ptregs );
  int  (* VendorInit)(int minor);

  uint8  (* InB)  (unsigned long adr);
  void   (* OutB) (uint8 v, unsigned long adr);

} can_atomic_func_t;





extern can_atomic_func_t *atomic_ops[];


/*---------- Primitives Wrapper */

#define CAN_ChipReset(minor)           (atomic_ops[minor]->ChipReset)(minor)
#define CAN_SetTiming(minor,baud)      (atomic_ops[minor]->SetTiming)(minor,baud)
#define CAN_StartChip(minor)           (atomic_ops[minor]->StartChip)(minor)
#define CAN_StopChip(minor)            (atomic_ops[minor]->StopChip)(minor)
#define CAN_SetMask(minor,arg)         (atomic_ops[minor]->SetMask)(minor,arg)
#define CAN_SetOMode(minor,arg)        (atomic_ops[minor]->SetOMode)(minor,arg)
#define CAN_SendMessage(minor,tx,wait) (atomic_ops[minor]->SendMessage)(minor,tx,wait)
#define CAN_GetMessage(minor,rx)       (atomic_ops[minor]->GetMessage)(minor,rx)
#define CAN_VendorInit(minor)          (atomic_ops[minor]->VendorInit)(minor)


/* this primitives are set by VendorInit */

#define CAN_Interrupt(minor)           atomic_ops[minor]->Interrupt
#define CAN_In_Func(minor)             atomic_ops[minor]->InB
#define CAN_Out_Func(minor)            atomic_ops[minor]->OutB


/*---------- Prototypes for memory model funcs --*/

extern uint8 PORT_In(unsigned long adr);
extern uint8 MEM_In(unsigned long adr);
extern void  PORT_Out(uint8 v,unsigned long adr);
extern void  MEM_Out(uint8 v,unsigned long adr);


}/

file "chipset" /{

can_atomic_func_t *atomic_ops[4];


void Set_IO_Model(int minor, char type) {
   switch(type){
	case 'm':
	   atomic_ops[minor]->InB = MEM_In;
	   atomic_ops[minor]->OutB = MEM_Out;
	   break;
	case 'p':
	   atomic_ops[minor]->InB = PORT_In;
	   atomic_ops[minor]->OutB = PORT_Out;
	   break;
        default:
	   printk("Can[%d]: Unknown memory model '%c' \n",minor,type);
           return;    	
   }
   IOModel[minor]=type;
}


uint8 PORT_In(unsigned long adr ) { 
        printk("PIn: 0x%x\n",adr);
        return inb(adr); 
}
uint8 MEM_In (unsigned long  adr ) { 
#if 0
        int ret=readb(adr);
        printk("MIn: 0x%x=0x%x\n",adr,ret);
        return ret;
#else
	return readb(adr);
#endif
}

void PORT_Out(uint8 v, unsigned long adr ) { 
#if IODEBUG
        printk("POut: 0x%x->0x%x\n",v,adr);
#endif
        outb(v,adr);  
}
void MEM_Out(uint8 v, unsigned long adr ) { 
#if IODEBUG
        printk("MOut: 0x%x->0x%x\n",v,adr);
#endif
        writeb(v,adr); 
}


}/



